# -*- coding: utf-8 -*-
'''
Created on 2010/10/07
 SHsSL LIST command.
@author: toru kageyama <info@comona.co.jp>
'''
# imports.
from settings import LINK_IGNORE_CASE
from util import trim

# LIST command.
def list(ftp, form, formatter, cur_dir = None):
    """LIST command."""
    files = []
    result = ListResult()
    if cur_dir == None:
        result.set_path_name(ftp.pwd()) # current directory
    else:
        result.set_path_name(cur_dir)   # current directory
    #list_dir = ftp.retrlines('LIST', files.append)
    list_dir = ftp.dir(files.append)
    for path in files:
        if path[0:5] == 'total':
            continue
        result.append(resolve_if_link(trim(path), ftp))

    # set result.
    formatter.append_result_data(result)
    formatter.set_command_result(True)

# entry point of first connection.
def connect(ftp, form, formatter):
    """entry point of first connection."""
    pwd = ftp.pwd()
    dir_list = []
    while True:
        dir_list.insert(0, pwd)
        if pwd == '/':
            break
        ftp.cwd('..')   # parent directory.
        pwd = ftp.pwd() # get directory name.

    for dir in dir_list:
        ftp.cwd(dir)
        list(ftp, form, formatter)

# resolve directory or file if link.
def resolve_if_link(path, ftp):
    """resolve directory or file if link."""
    pwd = ftp.pwd()
    if path[0] == 'l':
        new_path = resolve_link(path, ftp)
        ftp.cwd(pwd)
        return new_path
    else:
        return path

# resolve link.
def resolve_link(path, ftp):
    """resolve link."""
    arrow = path.rfind(' -> ')
    link = path[arrow + len(' -> '):]
    if len(link) < 1:
        return path
    path_delim = link.rfind('/')
    if path_delim == 0:
        # root directory.
        ftp.cwd('/')
    else:
        ftp.cwd(link[:path_delim])
    file_name = link[path_delim + 1:]
    if len(file_name) == 0:
        return 'D' + path[1:]   # link is root directory.

    # LIST directory
    files = []
    list_dir = ftp.retrlines('LIST', files.append)
    for new_path in files:
        if new_path[0:5] == 'total':
            continue
        if is_target_to_link(new_path, file_name) != False:
            if new_path[0] == 'd':
                return 'D' + path[1:]   # link is directory.
            elif new_path[0] == 'l':
                return resolve_link(new_path, ftp)  # link is link again.
            else:
                return path   # link is file
    return path

# examine the character string is target to file name or not.
def is_target_to_link(path, file_name):
    """examine the character string is target to file name or not."""
    pos = 0
    # file -rwx... mode.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # file count.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # file owner.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # file group.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # file size.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # file month.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # file day.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # file time or year.
    while True:
        if path[pos] == ' ':
            break;
        pos = pos + 1
    while True:
        if path[pos] != ' ':
            break;
        pos = pos + 1
    # now the top of the file name.
    new_path = path[pos:]
    if LINK_IGNORE_CASE:
        # MAC OSX link style?
        if new_path.lower() == file_name.lower():
            return True # file name matched.
        else:
            if len(new_path) > len(file_name):
                is_it_link = new_path[len(file_name):]
                new_path = new_path[:len(file_name)]
                if new_path.lower() == file_name.lower() and is_it_link.find(' -> ') == 0:
                    return True # another link.
    else:
        # LINUX link style?
        if new_path == file_name:
            return True # file name matched.
        else:
            if len(new_path) > len(file_name):
                is_it_link = new_path[len(file_name):]
                new_path = new_path[:len(file_name)]
                if new_path == file_name and is_it_link.find(' -> ') == 0:
                    return True # another link.
    return False

# LIST result class.    
class ListResult(object):
    """LIST result class."""

    # constructor.
    def __init__(self):
        """constructor."""
        self.files = []
        self.path_name = None

    # append.
    def append(self, obj):
        """append."""
        self.files.append(obj)

    # set path name.
    def set_path_name(self, path_name):
        """set path name."""
        self.path_name = path_name

    # get path name.
    def get_path_name(self):
        """get path name."""
        return self.path_name

    # print out.
    def print_out(self):
        """print out."""
        print '<list path="' + self.path_name + '">'
        for path in self.files:
            print '<path>' + path + '</path>'
        print '</list>'
